var _default_vert = 
`
attribute vec4 a_position;
attribute vec2 a_texCoord;
attribute vec4 a_color;
varying vec2 v_texCoord;
varying vec4 v_fragmentColor;
void main()
{
    gl_Position = CC_PMatrix * a_position;
    v_fragmentColor = a_color;
    v_texCoord = a_texCoord;
}
`;

var _gray_frag =
` 
varying vec2 v_texCoord;
void main()
{
    vec4 v = texture2D(CC_Texture0, v_texCoord).rgba;
    float f = v.r * 0.299 + v.g * 0.587 + v.b * 0.114;
    gl_FragColor = vec4(f, f, f, v.a);
}
`;

var _original_frag = 
` 
varying vec2 v_texCoord;
void main()
{ 
    gl_FragColor = texture2D(CC_Texture0, v_texCoord).rgba;
}
`;

cc.Class({
    extends: cc.Component,

    properties: {  
		childEnabled: {
            default: true, 
            tooltip: '内部所有子控件是否也生效',
            notify: function (oldValue) {
                if (CC_EDITOR && this.isValid && this.node.children.length > 0) {
                    if (oldValue) { 
                        this._initOriginProgram();  
                    } else {
                        this._initGrayProgram(); 
                    } 
                    this.node.children.forEach(element => {
                        this._setProgram(element._sgNode, oldValue ? this._originProgram : this._grayProgram);
                    });
                } 
            }
        }
    },
	
	editor: {
        requireComponent: cc.Sprite, 
        executeInEditMode: true
    },

    onEnable: function () {   
        this._initGrayProgram();
        if (this.childEnabled) {  
            this.node._sgNode.children.forEach(element => {
                this._setProgram(element, this._grayProgram);
            });
        } else {  
            this._setProgram(this.node.getComponent(cc.Sprite)._sgNode, this._grayProgram); 
            if (this.node.children.length > 0) {
                this._initOriginProgram(); 
                this.node.children.forEach(element => {
                    this._setProgram(element._sgNode, this._originProgram);
                });
            } 
        } 
    },

    onDisable: function () {  
        this._initOriginProgram();
        this.node._sgNode.children.forEach(element => {
            this._setProgram(element, this._originProgram);
        });
    }, 
    
    _initGrayProgram: function() {
        if (!this._grayProgram) {
            this._grayProgram = new cc.GLProgram();
            if (cc.sys.isNative) { 
                this._grayProgram.initWithString(_default_vert, _gray_frag); 
            } else {
                this._grayProgram.initWithVertexShaderByteArray(_default_vert, _gray_frag);
                this._grayProgram.addAttribute(cc.macro.ATTRIBUTE_NAME_POSITION, cc.macro.VERTEX_ATTRIB_POSITION);
                this._grayProgram.addAttribute(cc.macro.ATTRIBUTE_NAME_COLOR, cc.macro.VERTEX_ATTRIB_COLOR);
                this._grayProgram.addAttribute(cc.macro.ATTRIBUTE_NAME_TEX_COORD, cc.macro.VERTEX_ATTRIB_TEX_COORDS);
            }
            this._grayProgram.link();
            this._grayProgram.updateUniforms();
        }   
    },

    _initOriginProgram: function() {
        if (!this._originProgram) {
            this._originProgram = new cc.GLProgram();
            if (cc.sys.isNative) { 
                this._originProgram.initWithString(_default_vert, _original_frag); 
            } else {
                this._originProgram.initWithVertexShaderByteArray(_default_vert, _original_frag);
                this._originProgram.addAttribute(cc.macro.ATTRIBUTE_NAME_POSITION, cc.macro.VERTEX_ATTRIB_POSITION);
                this._originProgram.addAttribute(cc.macro.ATTRIBUTE_NAME_COLOR, cc.macro.VERTEX_ATTRIB_COLOR);
                this._originProgram.addAttribute(cc.macro.ATTRIBUTE_NAME_TEX_COORD, cc.macro.VERTEX_ATTRIB_TEX_COORDS);
            }
            this._originProgram.link();
            this._originProgram.updateUniforms();
        }   
    },

    _setProgram: function (sgNode, program) {
        if (cc.sys.isNative) { 
            sgNode.setGLProgramState(cc.GLProgramState.getOrCreateWithGLProgram(program));
        } else {
            sgNode.setShaderProgram(program);    
        } 
        sgNode.children.forEach(element => {
            this._setProgram(element, program);
        }); 
    }, 
});
